#pragma implementation
#include "AbsAutomata.h"
#include "BddPrint.h"

AbsAutomata::AbsAutomata(unsigned size, unsigned aSize) 
  : mySize(size), myAlphabetSize(aSize), myNumFinal(0), 
  myAccepting(size), myInitial(size) { ; }

AbsAutomata::~AbsAutomata() {;}

#ifdef NORMAL
void
AbsAutomata::printAut(FILE *output, const IntBddMap &labels, const AtomMap &idmap) const {
  fprintf(output, "%u\n", this->size());
  fputs("1\n", output);
  for(unsigned i=this->size(); i--; ) {
    fprintf(output, "%u ", i);
    if(this->isInitial(i))
      fputs("1 ", output);
    else 
      fputs("0 ", output);
    if(this->isFinal(i))
      fputs("0 ", output);
    fputs("-1\n", output);
    for(int j=this->alphabetSize(); j--; ) { 
      for(unsigned k=this->numArcs(i, j); k--;  ) {
	const class bdd &arc=labels[j];
	fprintf(output, "%u ", this->dest(i,j, k));
	printArc(arc, idmap, output);
      }
    }
    //    if(this->isFinal(i))
    //  fprintf(output, "%u t\n", i);
    fputs("-1\n", output);
  }
} 
#endif //normal


#ifdef MARIA
void
AbsAutomata::printAut(FILE *output, const IntBddMap &labels, const AtomMap &idmap) const {
  fprintf(output, "%u ", this->size());
  fputs("0\n", output);
  for(unsigned i=0; i<this->size(); i++) {
    fprintf(output, "%u ", i);
    if(this->isInitial(i))
      fputs("1 ", output);
    else if (this->isFinal(i))
      fputs("2 ", output);
    else
      fputs("0 ", output);
    fputs("-1\n", output);
    if(!this->isFinal(i)) {
      for(int j=this->alphabetSize(); j--; ) { 
	for(unsigned k=this->numArcs(i, j); k--;  ) {
	  const class bdd &arc=labels[j];
	  fprintf(output, "%u ", this->dest(i,j, k));
	  printArc(arc, idmap, output);
	}
      }
    }
    fputs("-1\n", output);
  }
} 
#endif //MARIA

#ifdef SPIN

void 
AbsAutomata::printAut(FILE *output, const IntBddMap &labels, const AtomMap &idmap ) const
{
  fputs("never {\n", output);
  for(unsigned i=0; i< this->size(); i++) {
    if(this->isInitial(i)) {
      fprintf(output, "T%u_init:\n", i);
    }
    else if (this->isFinal(i)) {
      fprintf(output, "accept_S%u:\n", i);
      fputs("\t assert(0)\n", output);
    }
    else {
      fprintf(output, "T0_S%u:\n", i);
    }
    if(!this->isFinal(i)) {
      fputs("\t if\n", output);
      for(int j=this->alphabetSize(); j--; ) { 
	for(unsigned k=this->numArcs(i, j); k--;  ) {
	  const class bdd &arc=labels[j];
	  fputs("\t :: ", output);
	  spinprintArc(arc, idmap, output);
	  fputs(" -> goto ", output);
	  if(this->isInitial(this->dest(i,j, k))) {
	    fprintf(output, "T%u_init\n", this->dest(i,j, k));
	  }
	  else if(this->isFinal(this->dest(i,j, k))) {
	    fprintf(output, "accept_S%u\n", this->dest(i,j, k));
	  }
	  else {
	    fprintf(output, "T0_S%u\n", this->dest(i,j, k));
	  }
	  
	}
      }
      fputs("\t fi;\n", output);
    }
  }
  fputs("}\n", output);
}
#endif //SPIN
