#ifndef AUTOMATA_H_
#define AUTOMATA_H_
#pragma interface
#include "typedef.h"

//forward decs
class NonDetAut;

class Automata {
 public:

  /**Pseudo constuructor for the class*/
  static class Automata *create(const class Formula &formula);
  /**@return The number of states in the automaton*/
  unsigned getSize() const { return mySize; }  
  /**Print the automata to a file
   * @param output The output stream where the automaton is printed
   * @param det should the automaton be determinzed & minimized
   * @param join should label joining optimisation be performed
   * @return Nondeterministic automaton in explisit form
   */
  NonDetAut* toNonDetAut(IntBddMap &labels, AtomMap &idmap) const;
  /**The destructor*/
  ~Automata() {}
  /**@return quantified variables*/
  class bdd getQuantified() const {return myQuantified;}
  /**@return unquantified variables*/
  class bdd getUnQuantified() const {return myUnQuantified;}

 private:
  /**The copy constructor*/
  Automata(const class Automata &other);
  /**The assignment operator*/
  const class Automata & operator=(const class Automata &other);

  /**Construct a new automaton object
   *@param formula Formula corresponding to automaton
   *@param transrel The transition relation of the automaton
   *@param statespace The reachable states of the automaton
   */
  Automata(const class Formula &formula, const class bdd & transrel, 
	   const class bdd &statespace, const class bdd &quantified, 
	   const class bdd &unquantified, unsigned size) : 
    myFormula(formula), myTransRel(transrel), 
    myStateSpace(statespace), myQuantified(quantified), 
    myUnQuantified(unquantified), mySize(size) { ;} 

 private:
  const class Formula &myFormula;
  const class bdd myTransRel;
  const class bdd myStateSpace;
  const class bdd myQuantified;
  const class bdd myUnQuantified;
  unsigned mySize;
};
#endif //AUTOMATA_H_ 


