#include "BddPrint.h"
#include "Atom.h"

void
printPath(const BddMap &bddmap, FILE* output)
{
  unsigned j=bddmap.size();
  for(BddMap::const_iterator i=bddmap.begin(); i!=bddmap.end(); ++i) {
    if(--j)
      fputs("& ", output);
    if((*i).second.second) 
      fprintf(output, "p%d ", (*i).second.first);
    else 
      fprintf(output, "! p%d ", (*i).second.first);
  }
}
#include<vector>
typedef vector<Atom> AtomMap;

void
printBDD(const class bdd &arc, FILE* output, const AtomMap &idmap, BddMap &bddmap, int & pathcount, int calls)
{
  if( calls == 0 && pathcount > 1) {
    fprintf(output, "| "); 
    pathcount--;
  }

  int tratt=bdd_var(arc);
  if( (bdd_high(arc) != bdd_false()) && 
      bddmap.find(bdd_high(arc)) == bddmap.end() ) {
    if(bdd_high(arc)==bdd_true()) {
      bddmap[arc]=BddMap::data_type(idmap[tratt].getId(), idmap[tratt].getSign());
      printPath(bddmap, output);
      if (--pathcount > 0)
	fputs("| ", output);
    }
    else {
      bddmap[arc]=BddMap::data_type(idmap[tratt].getId(), idmap[tratt].getSign());
      printBDD(bdd_high(arc), output, idmap, bddmap, pathcount, ++calls);
    }
  }
  if((bdd_low(arc) != bdd_false()) && 
     bddmap.find(bdd_low(arc)) == bddmap.end() ) {
    if (bdd_low(arc)==bdd_true()) {
      bddmap[arc]=BddMap::data_type(idmap[tratt].getId(), !idmap[tratt].getSign());
      printPath(bddmap, output);
      if (--pathcount > 0)
	fputs("| ", output);
    }
    else {
      bddmap[arc]=BddMap::data_type(idmap[tratt].getId(), !idmap[tratt].getSign());
      printBDD(bdd_low(arc), output, idmap, bddmap, pathcount, ++calls);
    } 
  }
  bddmap.erase(arc);
}

/**
 * Print the arc to a stream
 * @param arc BDD represting the arc 
 * @param idmap Map of bdd var nums to ap id:s
 * @param output stream where the arc is printed
 */


void 
printArc(const class bdd &arc, const AtomMap &idmap, FILE *output) {
  if(arc == bdd_true()) {
    fputs("t\n", output);
    return;
  }
  BddMap bddmap;
  int pathcount=static_cast<int>(bdd_pathcount(arc));
  printBDD(arc, output, idmap, bddmap, pathcount);
  fputs("\n", output);
}


inline void 
helpprint(const class bdd &arc, const AtomMap &idmap, FILE *output, bool val) 
{
  if(val) {
    if(idmap[bdd_var(arc)].getSign())
      fprintf(output, "p%u ", idmap[bdd_var(arc)].getId());
    else
      fprintf(output, "! p%u", idmap[bdd_var(arc)].getId());
  }
  else {
    if(idmap[bdd_var(arc)].getSign())
      fprintf(output, "! p%u", idmap[bdd_var(arc)].getId());
    else
      fprintf(output, "p%u", idmap[bdd_var(arc)].getId());
  }
}


/**Print an arc to a stream in the format of spin
 * @param arc BDD represting the arc 
 * @param idmap Map of bdd var nums to ap id:s
 * @param output stream where the arc is printed
 */

  
void spinprintArc(const class bdd &arc, const AtomMap &idmap, FILE *output)
{
  if (arc==bdd_true()) {
    fputs(" (1) ", output);
    return;
  }
  fputs("(", output);
  if (bdd_high(arc)!= bdd_false()) {
    fputs("(", output);
    helpprint(arc, idmap, output, true);
    if (bdd_high(arc)!=bdd_true()) { 
      fputs(" && ", output);
      spinprintArc(bdd_high(arc), idmap, output);
    }
    fputs(")", output);
  }
  if(bdd_low(arc)!=bdd_false()) {
    if(bdd_high(arc)!=bdd_false()) {fputs(" || ", output);} 
    fputs("(", output);
    helpprint(arc, idmap, output, false);
    if (bdd_low(arc)!=bdd_true()) {
      fputs(" && ", output);
      spinprintArc(bdd_low(arc), idmap, output);
    }
    fputs(")", output);
  }
  fputs(")", output);
  return;
} 
