#pragma interface
#ifndef BUCHI_H_
#define BUCHI_H_
#include "bdd.h"
#include "typedef.h"

//forward def of ExplicitBuchi;
class ExplicitBuchi;

#include <slist>
typedef std::slist<class bdd> bddList;

class Buchi {

 public:
  /**Create a new Buchi automaton representing a formula*/
  static class Buchi* create(const class Formula &f);
  /**Determine if the automaton represents a safety language*/
  bool isSafety();
  
  /**The destructor*/
  ~Buchi();

  /**Convert the symbolic representations to an explicit one
   *@param labels (output) Labels for the arcs of the automaton  
   *@param idmap (output) Names for the atomic propositions
   *@return The explicit automaton corresponding to the symbolic Buchi automaton
   */
  
  class ExplicitBuchi *toExplicit(IntBddMap &labels, AtomMap &idmap) const;

 private:
  /**Constructor*/
  Buchi(const class Formula &formula, 
	class bdd &transRel, class bdd &statespace, 
	bddList *fairness, class bdd &initial);
  
  const class Formula &myFormula;
  class bdd myTransRel;
  class bdd myStateSpace;
  bddList *myFairness;
  class bdd myInitial;
};


#endif //BUCHI_H_
