#ifndef DETAUT_H_
#define DETAUT_H_
#pragma interface
#include "AbsAutomata.h"
#include <cassert>

//forward def of NonDetAut
class NonDetAut;

class DetAut : public AbsAutomata  {

 public:
  /**Constructor of the class
   * @param size The number of states in the automaton
   * @param aSize The size of the alphabet
   */
  DetAut(unsigned size, unsigned aSize);
  /**The destructor*/
  ~DetAut() {
    delete[] myTransRel;
  }
  /**Add transition
   * @param source Id of the source state
   * @param label Label of the transition
   * @param dest Id of the destination state
   */
  void addTransition(unsigned source, unsigned label, unsigned dest) {
    assert(source<mySize && label<myAlphabetSize && dest<mySize);
    myTransRel[source*myAlphabetSize + label]=++dest;  
  }
  /**Delete a transition
   *@param source The state where the edge leaves
   *@param label The label of the edge to be deleted
   */
  void delTransition(unsigned source, unsigned label, unsigned dest = 0) {
    assert(source < mySize && label<myAlphabetSize);
    myTransRel[source*myAlphabetSize + label]=0;  
  }
  /** 
   *@param source start state of the edge
   *@param label Label of the edge
   *@return the destination of a given edge
   */
  unsigned dest(unsigned source, unsigned label, unsigned index = 0) const {
    assert(source < mySize && label<myAlphabetSize && index==0);
    return myTransRel[source*myAlphabetSize + label] - 1;  
  }
  /**
   *@param source Start state of the edges
   *@param label Label of the edges
   *@return Number of edges from a state with a given label
   */
  unsigned numArcs(unsigned source, unsigned label) const {
    assert(source<mySize && label<myAlphabetSize);
    return (myTransRel[source*myAlphabetSize + label]) ? 1 : 0;   
  }
  
  /**Construct a minimized automoton from this one. Remember to delete 
   *the result!
   *@return Minimized automaton
   */
  class DetAut* minimize() const;
  /**Set the size of the automaton to size and allocate more memory if necessary*/
  void setSize(unsigned size);

  /**Set the size of the alphabet of the automaton
   *@param size The new size of the alphabet 
   */
  void setAlphabetSize(unsigned size);
  
  /**Interpret this automaton as a Buchi automaton and
   * return its complement. Remember to deallocate the result.
   * If the parameter True==myAlphabetSize, the algoritm assumes that
   * 'true' was not present among the labels and added. 
   * @param True label index for the 'true' label
   * @return Buchi complement of this automaton
   */
  class NonDetAut *buchiComplement(unsigned True) const;

  /** Join labels which have the same destination. The procedure destroys 
   * this automaton.
   *@param labels Table containing the labels in bdd form
   *@return The resulting non deterministic automaton
   */
  class NonDetAut *labelJoin(IntBddMap &labels);

 private:
  /**The transition relation*/
  unsigned *myTransRel;
  /**The number of words allocated for the states*/
  unsigned myAllocated;
};

#endif //DETAUT_H_

