#pragma implementation
#include <cstdlib>
#include "NonDetAut.h"


NonDetAut::NonDetAut(unsigned size, unsigned aSize) 
  : AbsAutomata(size, aSize), myAllocated(aSize)
{
  assert(size>0);
  myTransRelSize=new unsigned[size*aSize];
  myTransRel=new unsigned*[size*aSize];
  memset(myTransRelSize, 0, size*aSize * sizeof (unsigned));
  memset(myTransRel, 0, size*aSize * sizeof (unsigned));
}

NonDetAut::~NonDetAut() 
{
  for (unsigned i=mySize*myAllocated; i--; )
    delete[] myTransRel[i];
  delete[] myTransRel;
  delete[] myTransRelSize;
}

void
NonDetAut::setASize(unsigned size) 
{
  if (size<=myAllocated) {
    myAlphabetSize=size;
    return;
  }
  unsigned oldSize=myAllocated;
  while(size >= myAllocated) {++myAllocated; myAllocated *= 2;}
  unsigned *temp=new unsigned[mySize*myAllocated];
  memcpy(temp, myTransRelSize, mySize*oldSize*sizeof (unsigned));
  memset(temp + mySize*oldSize, 0, mySize*(myAllocated-oldSize)*sizeof(unsigned)); 
  delete[] myTransRelSize;
  myTransRelSize=temp;
  unsigned **temporary=new unsigned*[mySize*myAllocated];
  memcpy(temporary, myTransRel, mySize*oldSize*sizeof (unsigned));
  memset(temporary + mySize*oldSize, 0, mySize*(myAllocated-oldSize)*sizeof(unsigned)); 
  delete[] myTransRel;
  myTransRel=temporary;
  myAlphabetSize=size;
}

void 
NonDetAut::addTransition(unsigned source, unsigned label, unsigned dest) {
  assert(source<mySize && label<myAlphabetSize && dest<mySize);
  unsigned *temp=new unsigned[myTransRelSize[label*mySize + source]+1];
  memcpy(temp, myTransRel[label*mySize + source],
	 myTransRelSize[label*mySize + source] * sizeof(unsigned));
  delete[] myTransRel[label*mySize + source];
  temp[myTransRelSize[label*mySize + source]++]=dest;
  myTransRel[label*mySize + source]=temp;
}

#include <hash_map>
#include <slist>
struct eqbv
{
  bool operator()(const BitVector &s1, const BitVector &s2) const
  {
    return s1==s2;
  }
};
struct bvhasher
{
  int operator()(const BitVector &s) const
  {
    int hvalue=246093032;    
    return s.hash(hvalue);
  }
};

typedef std::hash_map<class BitVector, unsigned, bvhasher, eqbv> PowerSet;
typedef std::slist<PowerSet::const_iterator> IterList;

/**Compute the transitive closure w.r.t a label from a set of states
 *@param aut The automaton where working in
 *@param start The set of states 
 *@param label The label 
 *@param result place holder for the result
 */

inline static void
closure(const class AbsAutomata &aut, const class BitVector &start, unsigned label, class BitVector &result)
{
  for(unsigned i=start.getSize(); i--; ) 
    if(start[i]) 
      for (unsigned j=aut.numArcs(i, label); j--; )
	result.assign(aut.dest(i, label, j), true);
}

#include "DetAut.h"

void 
NonDetAut::determinize(class DetAut &result) const
{
    PowerSet statespace;
    unsigned state=0;
    PowerSet::const_iterator i=(statespace.insert(PowerSet::value_type(myInitial, state))).first;  
    result.makeInitial(0);
    IterList work;
    work.push_front(i);
    do {
      PowerSet::const_iterator iter=work.front();
      work.pop_front();
      const BitVector &source=(*iter).first;
      for (unsigned j=myAlphabetSize; j--; ) { //loop over all labels
        BitVector dest(mySize);
	closure(*this, source, j, dest);	  
        if (statespace.find(dest)==statespace.end()) { //seen before?
	  if((state+1) >= result.size()) result.setSize(result.size()+1);	
	  result.addTransition((*iter).second ,j, ++state);
	  work.push_front((statespace.insert(PowerSet::value_type(dest, state))).first);	 
	  if (dest.andPos(myAccepting))
	    result.makeFinal(state);	
	}	
	else
	  result.addTransition((*iter).second ,j, (*statespace.find(dest)).second);
      }    
    }while (!work.empty());
    result.setSize(state+1);
}

#include "typedef.h"

inline static void
symClosure(const NonDetAut &aut, const BitVector &source, const IntBddMap &labels, 
	   const class bdd &label, class BitVector &dest)
{
  for (unsigned i=source.getSize(); i--; ) {
    if (!source[i]) continue;
    for(unsigned j=aut.alphabetSize(); j--; ) {
      if (!aut.numArcs(i,j)) continue;
      if ((label & labels[j]) == bdd_false()) continue;
      for (unsigned k=aut.numArcs(i,j); k--; ) {
	dest.assign(aut.dest(i,j,k) ,true);	
      }
    }
  }
}


typedef std::hash_map<unsigned, class bdd> IntToBddMap;

DetAut*
NonDetAut::symbolicDet(const IntBddMap &labels, IntBddMap &newLabels, class bdd &apvars) const   
{
  DetAut *result=new DetAut(1, 0);
  PowerSet statespace;
  BddMap labelToInt;
  unsigned state=0;
  unsigned labelCount=0;
  PowerSet::const_iterator i=(statespace.insert(PowerSet::value_type(myInitial, state))).first;  
  result->makeInitial(0);
  IterList work;
  work.push_front(i);
  do {
    PowerSet::const_iterator iter=work.front();
    work.pop_front();
    const BitVector &source=(*iter).first;
    IntToBddMap tempLabel;
    class bdd usedAP=bdd_true();
    for (class bdd label=bdd_satoneset(usedAP, apvars, bdd_false()); 
	 usedAP!=bdd_false(); usedAP &= !label, label = bdd_satoneset(usedAP, apvars, bdd_false())) { //loop over all labels
      BitVector dest(mySize);
      symClosure(*this, source, labels, label, dest);
      PowerSet::const_iterator destIter=statespace.find(dest);
      if (destIter==statespace.end()) { //seen before?
	if((state+1) >= result->size()) result->setSize(result->size()+1);
	destIter=statespace.insert(PowerSet::value_type(dest, ++state)).first;
	work.push_front(destIter);	
	if (dest.andPos(myAccepting)) result->makeFinal(state);	
      }	
      IntToBddMap::iterator labelIter=tempLabel.find((*destIter).second);
      if(labelIter==tempLabel.end()){ 
	tempLabel.insert(IntToBddMap::value_type((*destIter).second, label));
      }
      else {
	(*labelIter).second |= label;
      }
    }
    for(IntToBddMap::iterator labelIter=tempLabel.begin(); labelIter!=tempLabel.end(); ++labelIter ) {
      BddMap::const_iterator prevIter=labelToInt.find((*labelIter).second);
      if (prevIter!=labelToInt.end()) {
	result->addTransition((*iter).second, (*prevIter).second.first, (*labelIter).first);
      }
      else {
	result->setAlphabetSize(labelCount+1);
	result->addTransition((*iter).second, labelCount, (*labelIter).first);
	labelToInt[(*labelIter).second]=BddMap::data_type(labelCount++, false);
	newLabels.push_back((*labelIter).second);
	assert(newLabels.size() == labelCount);
      }
    }
  }while (!work.empty());
  result->setSize(state+1);
  return result;
}


