//Class for Non-deterministic automaton
#ifndef NONDETAUT_H_
#define NONDETAUT_H_
#pragma interface
#include "AbsAutomata.h"
#include <cassert>

class NonDetAut : public AbsAutomata  {

 public:
  /**Constructor of the class
   * @param size The number of states in the automaton
   * @param aSize The size of the alphabet
   */
  NonDetAut(unsigned size, unsigned aSize);
  /**The destructor*/
  ~NonDetAut();
  /**Add transition
   * @param source Id of the source state
   * @param label Label of the transition
   * @param dest Id of the destination state
   */
  void addTransition(unsigned source, unsigned label, unsigned dest);
  /**Delete a transition
   * @param source Id of the source state
   * @param label Label of the transition
   * @param dest Id of the destination state
   */
  void delTransition(unsigned source, unsigned label, unsigned dest) {
    assert(source <= mySize && label<myAlphabetSize && dest<mySize);
    unsigned i=0;
    for(; dest!=myTransRel[label*mySize + source][i]; i++);
    unsigned temp=myTransRel[label*mySize + source][i];
    myTransRel[label*mySize + source][i]
      =myTransRel[label*mySize + source][myTransRelSize[label*mySize + source]];
    myTransRel[label*mySize + source][myTransRelSize[label*mySize + source]--]=temp;
  }
  /**Return the name of a target state. The method does not fail even if the
   * requested destination does not exist. Use the method numArcs to check
   * the given result is correct.
   *@param source state
   *@param label The arc label
   *@index Which arc
   *@return The name of target state
   */
  unsigned dest(unsigned source, unsigned label, unsigned index) const {
    assert(source < mySize && label<myAlphabetSize && 
	   index<myTransRelSize[label*mySize + source]);
    return myTransRel[label*mySize + source][index];  
  }  
  /**@return Number of arcs with the given source and label*/  
  unsigned numArcs(unsigned source, unsigned label) const {
    assert(source<mySize && label<myAlphabetSize);
    return myTransRelSize[label*mySize + source];   
  }
  /**Determinze this nondet automaton
   *@param result Place holder for the result
   */
  void determinize(class DetAut &result) const;
  /**Symbolic determinisation of the automaton
   *@param labels The labels in bdd form
   *@param newLabels (output) The labels of the resulting deterministic automaton
   *@param apvars A bdd with the variables corresponding to the relevant atomic props
   *@return The deterministic automaton
   */
  DetAut *symbolicDet(const IntBddMap &labels, IntBddMap &newLabels, 
		      class bdd & apvars) const;

  /**Increase the size of the alphabet*/
  void setASize(unsigned size);
  
 private:
  /**The transition relation*/
  unsigned *myTransRelSize;
  unsigned **myTransRel;
  /**Allocated alphabet size*/
  unsigned myAllocated;
};

#endif //NONDETAUT_H_

