// Class for representing the negation of a formula -*- c++ -*-
#ifndef NOT_H_
#define NOT_H_
#pragma interface

#include "Formula.h"

class Not : public Formula {

 public:
  /**Construct a negated formula
   *@param formula Formula to be negated
   */
  Not(const class Formula *formula);
  /**The destructor*/
  ~Not() {; }
  void destroy() {
    const_cast<Formula *>(myFormula)->destroy();
    delete this;
  }
  /**The copy constructor*/
  Not(const class Not &other);
  /**@return A deep copy of this object*/
  Not* clone() const {
    return new Not(*this);
  }
  /**The assignment operator*/
  class Not & operator=(const class Not &other);
  /**@return The operand*/
  const class Formula *getOperand() const {return  myFormula;}
  class Formula *getOperand() {return const_cast<Formula *>(myFormula);}
  
  /**Print the formula in post-fix order to file*/
  void print (FILE *file) const;
  /**@return The type of formula*/
  enum Formula::Type getType() const { return fNot;} 
  /**@return hashvalue of the formula*/
  int hash() const {return 7+10*myFormula->hash();}
  /**@return true if the other formula is syntactically equal*/
  bool operator==(const class Formula &other) const;

 private:
  //the negated formula
  const class Formula *myFormula;  
};

#endif //NOT_H_
