// Main file of the automata generator -*- c++ -*-
#include <cstdio>
#include <getopt.h>
#include "Formula.h"
#include "FormulaAlgs.h"


/**Open the files
 *@param inputFile The file to be read from
 *@param outputFile The file to be written to
 *@return false upon failure
 */
inline static bool 
openFiles(FILE *&inputFile, FILE *&outputFile, char **argv, int argc) 
{
  inputFile=fopen(argv[argc-2], "r");
  if(inputFile==NULL) {
    fprintf(stderr, "Could not open file %s\n", argv[argc-2]);
    return false;
  }
  outputFile=fopen(argv[argc-1], "w+");  
  if(outputFile==NULL) {
    fprintf(stderr, "Could not open file %s\n", argv[argc-1]);
    return false;
  }
  return true;  
}


/**Print the help to stderr and the given message
 * @param message The message to be printed
 */
inline static void 
printHelp(const char *message) 
{
 fputs(message, stderr);
 fputs("Usage: scheck {-options} inputfile outputfile\n", stderr);
 fputs("-m \t Deterministic \n-t [translator] \t Define translator\n", stderr);
 fputs("-s\t Reject syntactically unsafe formulas\n-b \t Generate a Buchi automaton\n", stderr);   
 fputs ("-s \t Reject pathologic formulas\n", stderr);
 fputs("-c\t Generate the buchi complement of the safety automaton\n", stderr);
}

#include "Automata.h"
#include "Buchi.h"
#include "ExplicitBuchi.h"
#include "Pathologic.h"
#include "NonDetAut.h"
#include "DetAut.h"


//  ExplicitBuchi* testAut(IntBddMap &labels, AtomMap &idmap)
//  {
//    if (!bdd_isrunning()) bdd_init(500000, 50000);
//    if (static_cast<unsigned>(bdd_varnum()) < 26) bdd_setvarnum(26);

//    //labels
//    labels.push_back(bdd_true());
//    labels.push_back(bdd_nithvar(2)); //!p1
//    labels.push_back(bdd_nithvar(8)); //!p2
//    labels.push_back(bdd_ithvar(8));  //p2
//    labels.push_back(bdd_nithvar(12)); //!p3
//    labels.push_back(bdd_nithvar(2) & bdd_nithvar(12)); //!p1&&!p3
//    labels.push_back(bdd_ithvar(24)); //p4

//    //map bddvarnums to ids
//    idmap.reserve(25);
//    idmap[2]=class Atom(1);
//    idmap[8]=class Atom(2);
//    idmap[12]=class Atom(3);
//    idmap[24]=class Atom(4);

//    class ExplicitBuchi *aut = new ExplicitBuchi(15, 7, 1);
//    //transitions
//    // (0, a, x)
//    aut->addTransition(0, 6, 10);
//    aut->addTransition(0, 0, 9);
//    aut->addTransition(0, 1, 1);
//    aut->addTransition(0, 5, 2);
//    aut->addTransition(0, 5, 8);
//    aut->addTransition(0, 1, 7);
//    aut->addTransition(0, 0, 5);
//    aut->addTransition(0, 4, 6);
//    aut->addTransition(0, 5, 4);
//    aut->addTransition(0, 4, 3);
//    //(10, a, x)
//    aut->addTransition(10, 0, 10);
//    //(9, a, x)
//    aut->addTransition(9, 0, 9);
//    aut->addTransition(9, 6, 10);
//    //(1,a, x)
//    aut->addTransition(1, 0, 1);
//    aut->addTransition(1, 4, 2);
//    //(2, a, x)
//    aut->addTransition(2, 0, 2);
//    aut->addTransition(2, 0, 13);
//    //(8, a, x)
//    aut->addTransition(8, 0, 8);
//    aut->addTransition(8, 1, 13);
//    aut->addTransition(8, 4, 14);
//    //(7,a,x)
//    aut->addTransition(7, 0, 7);
//    aut->addTransition(7, 5, 13);
//    aut->addTransition(7, 1, 12);
//    aut->addTransition(7, 4, 8);
//    //(5, a, x)
//    aut->addTransition(5, 0, 5);
//    aut->addTransition(5, 5, 13);
//    aut->addTransition(5, 1, 7);
//    aut->addTransition(5, 1, 12);
//    aut->addTransition(5, 5, 8);
//    aut->addTransition(5, 1, 14);
//    aut->addTransition(5, 4, 6);
//    aut->addTransition(5, 4, 11);
//    //(6,a,x)
//    aut->addTransition(6, 0, 6);
//    aut->addTransition(6, 1, 8);
//    aut->addTransition(6, 5,13);
//    aut->addTransition(6, 5,14);
//    aut->addTransition(6, 4,11);
//    //(4, a, x)
//    aut->addTransition(4, 0, 4);
//    aut->addTransition(4, 3,14);
//    //(3,a,x)
//    aut->addTransition(3, 0, 3);
//    aut->addTransition(3, 1, 4);
//    //(13, a, x)
//    aut->addTransition(13, 2, 13);
//    aut->addTransition(13, 2, 2);
//    //(12, a, x)
//    aut->addTransition(12, 0, 12);
//    aut->addTransition(12, 4, 13);
//    //(14, a, x)
//    aut->addTransition(14, 3, 14);
//    aut->addTransition(14, 0, 4);
//    //(11, a, x)
//    aut->addTransition(11, 0,11);
//    aut->addTransition(11, 1,14);
    
//    //final states
//    aut->makeFinal(10, 0);
//    aut->makeFinal(13, 0);
//    aut->makeFinal(14, 0);
//    //inital state
//    aut->makeInitial(0);
//    return aut;
//  }

/**Initialise the complement
 * @param detaut Deterministic automaton to be complemented
 * @param labels Labels for the automaton
 * @return Complement of detaut interpreted as Buchi automaton
 */

inline static NonDetAut *
initComp(const DetAut &detaut, IntBddMap &labels) 
{
  unsigned i=0;
  for(  ; i<labels.size(); i++) {
    if(labels[i]==bdd_true()) 
      break;
  }
  if (i==labels.size()) labels.push_back(bdd_true());
  return detaut.buchiComplement(i);
}

/**Find the index of "true". If it does not exist return labels.size().
 *@param labels Labels of an automaton
 *@return the index of "true" 
 */


inline static unsigned 
findTrue(IntBddMap &labels) {
  unsigned i=0;
  for(  ; i<labels.size(); i++) {
    if(labels[i]==bdd_true()) { 
      break;
    }
  }
  return i;
}

bdd labelBDD(const class Formula &f);

/**Main function. Parses options and acts according to the options. */

extern char *optarg;
extern int optopt;
void removeSink(class DetAut &, unsigned True);

int main(int argc, char** argv) {
  /**Pointer to input file*/
  FILE *inputFile;
  /**Pointer to output file*/
  FILE *outputFile;
  
  if(argc<3) {printHelp("Wrong number of arguments!\n"); return 1;}

  const char *translator;
  bool det=false;
  bool trans=false;
  bool reject=false;
  bool pathologic=false;
  bool buchi=false;
  bool comp=false;
  for(;;) {
    int c=getopt(argc, argv, "cbmt:sp");
    if(c==-1) break; //no more options
    switch(c) {
    case 'c':
      if(buchi || det) { 
	printHelp("Only one of the options c,b,m, may be enabled at a time\n");
	return 1;
      }
      comp=true;
      break;
    case 'b':
      if(det || comp) { 
	printHelp("Only one of the options c,b,m, may be enabled at a time\n");
	return 1;
      }
      buchi=true;
      break;
    case 'm':
      if(buchi || comp) { 
	printHelp("Only one of the options c,b,m, may be enabled at a time\n");
	return 1;
      }
      det=true;
      break;
    case 't':
      translator=optarg;
      trans=true;
      break;
    case 's':
      reject=true;
      break;
    case 'p':
      pathologic=true;
      break;
    case ':':
      fputs("Missing argument for the translator\n", stderr);
      return 1;
    case '?':
      fprintf(stderr, "Unknown option %c.\n", optopt);
      return 1;
    default:
      fputs("Error parsing options.\n", stderr);
      return 1;
   }
  }

  if(!openFiles(inputFile, outputFile, argv, argc)) return 3;

  class Formula *f=0;
  if(!(f=parseFormula(inputFile))) return 4;
  class Formula *f2=removeDerived(f);
  f->destroy();      
  class Formula *f3=toNNF(f2);
 
  /**Error code returned*/
  int error=0;

  if (reject && !isSyntacticSafe(*f3)) { //check for syntactic safe
    fputs("Formula is not syntactically safe.\n", stderr); 
    error=5;
  } 
  
  if(!error && buchi) { //output buchi automaton
    class Buchi *aut;
    class ExplicitBuchi *expaut;
    IntBddMap labels; AtomMap idmap;
    if(!trans) {
      aut=Buchi::create(*f3);
      expaut=aut->toExplicit(labels, idmap);
      delete aut;
    }
    else {
      expaut=buchiRead(*f2, translator, labels, idmap);     
    }
    expaut->printAut(outputFile, labels, idmap);
    delete expaut;
  }

  if(!error && !buchi) {
    class Automata *aut=Automata::create(*f3);
    if(aut) {
      IntBddMap labels;
      AtomMap idmap;
      NonDetAut *expaut=aut->toNonDetAut(labels, idmap);
      delete aut;
      if(reject && !det && !comp) { // default behaviour, output resulting automaton
	expaut->printAut(outputFile,labels, idmap);	
      }
      else {
	IntBddMap newLabels;
	class bdd ap=labelBDD(*f3);
	DetAut *detaut=expaut->symbolicDet(labels, newLabels, ap);
	if (comp) { //output complement
	  NonDetAut *complement=initComp(*detaut, newLabels);
	  complement->printAut(outputFile, newLabels, idmap);
	  delete complement;
	}
	else { //output deterministic automaton 
	  if(pathologic) {
	    Pathologic path(*f3, *detaut, (trans) ? translator : 0);	  
	    if(!path.pathologic(newLabels)) {
	      fputs("The formula is pathologic or a liveness formula!\n", stderr);
	      error=7;
	    }
	  }    
	  if (!error) {
	    removeSink(*detaut, findTrue(newLabels));
	    if (det) detaut->printAut(outputFile, newLabels, idmap);
	    else expaut->printAut(outputFile,labels, idmap);
	  }	  
	}
	delete expaut;   
	delete detaut;
      }
    }
    else {
      error=6;
      fputs("Formula is not a safety formula.\n", stderr);
    }
  } 
 
  f2->destroy();
  f3->destroy();
  bdd_done();
  fclose(inputFile);
  fclose(outputFile);
  return error;
}






